#!/usr/bin/env python

"""
MIT License

Copyright (c) 2020-2021 Linus Bartsch

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from dataclasses import dataclass
from enum import Enum
from typing import Union, List


class ApplicationCommandOptionType(Enum):
    SUB_COMMAND = 1
    SUB_COMMAND_GROUP = 2
    STRING = 3
    INTEGER = 4
    BOOLEAN = 5
    USER = 6
    CHANNEL = 7
    ROLE = 8


@dataclass()
class ApplicationCommandOptionChoice:
    name: str
    value: Union[str, int]

    def to_dict(self):
        return {"name": self.name, "value": self.value}


@dataclass()
class ApplicationCommandOption:
    type: ApplicationCommandOptionType
    name: str
    description: str
    default: bool = False
    required: bool = False
    choices: List[ApplicationCommandOptionChoice] = None
    options: List["ApplicationCommandOption"] = None

    @classmethod
    def from_dict(cls, data) -> "ApplicationCommandOption":
        option_type = ApplicationCommandOptionType(data.pop("type"))
        data["choices"] = [
            ApplicationCommandOptionChoice(**choice)
            for choice in data.get("choices", [])
        ] or None
        data["options"] = [
            cls.from_dict(option) for option in data.get("options", [])
        ] or None
        return cls(type=option_type, **data)

    def to_dict(self) -> dict:
        data = {
            "type": self.type.value,
            "name": self.name,
            "description": self.description,
            "default": self.default,
            "required": self.required,
        }

        if self.choices:
            data["choices"] = [choice.to_dict() for choice in self.choices]
        if self.options:
            data["options"] = [option.to_dict() for option in self.options]

        return data


class ApplicationCommand:
    def __init__(
        self,
        name,
        description,
        options: List[ApplicationCommandOption] = None,
        **kwargs
    ):
        self.id = int(kwargs.get("id", 0)) or None
        self.application_id = int(kwargs.get("application_id", 0)) or None
        self.name = name
        self.description = description
        self.options = options

    def add_option(self, option: ApplicationCommandOption):
        if self.options is None:
            self.options = [option]
        else:
            self.options.append(option)

    @classmethod
    def from_dict(cls, data) -> "ApplicationCommand":
        data["options"] = [
            ApplicationCommandOption.from_dict(option)
            for option in data.get("options", [])
        ] or None
        return cls(**data)

    def to_dict(self) -> dict:
        data = {"name": self.name, "description": self.description}

        if self.id:
            data["id"] = self.id
            data["application_id"] = self.application_id
        if self.options:
            data["options"] = [option.to_dict() for option in self.options]

        return data



#!/usr/bin/env python

from .interaction import (
    Interaction,
    InteractionType,
    ApplicationCommandInteractionData,
    ApplicationCommandInteractionDataOption,
)

from .interaction_response import (
    InteractionResponse,
    InteractionResponseType,
    InteractionApplicationCommandCallbackData,
    FollowupMessage,
)

from .application_command import (
    ApplicationCommand,
    ApplicationCommandOption,
    ApplicationCommandOptionType,
    ApplicationCommandOptionChoice,
)

from .member import Member, User, UserFlags, PremiumType
from .client import ApplicationClient, InteractionClient
from .utils import verify_key

from . import flask_ext, ocm



#!/usr/bin/env python

"""
MIT License

Copyright (c) 2020-2021 Linus Bartsch

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from flask import Flask, request, jsonify
from discord_interactions import (
    Interaction,
    InteractionType,
    InteractionResponse,
    InteractionResponseType,
    InteractionApplicationCommandCallbackData,
    verify_key,
    ApplicationCommand,
    ApplicationClient,
)
from discord_interactions import ocm
from typing import Callable, Union, Type, Dict, List


_CommandCallback = Callable[
    [Union[Interaction, ocm.Command]], Union[InteractionResponse, str, None]
]


class Interactions:
    def __init__(self, app: Flask, public_key: str):
        self._app = app
        self._public_key = public_key

        app.add_url_rule("/", "interactions", self._main, methods=["POST"])

        self._commands: Dict[str, ApplicationCommand] = {}
        self._callbacks: Dict[str, _CommandCallback] = {}

    @property
    def commands(self) -> List[ApplicationCommand]:
        """ All registered application commands """

        return list(self._commands.values())

    def create_commands(self, client: ApplicationClient, guild: int = None):
        """ Create all registered commands as application commands at Discord. """

        for cmd in self.commands:
            client.create_command(cmd, guild=guild)

    def _verify_request(self):
        signature = request.headers.get("X-Signature-Ed25519")
        timestamp = request.headers.get("X-Signature-Timestamp")

        if signature is None or timestamp is None:
            return False

        return verify_key(request.data, signature, timestamp, self._public_key)

    def _main(self):
        # Verify request
        if not self._app.config["TESTING"]:
            if not self._verify_request():
                return "Bad request signature", 401

        # Handle interactions
        interaction = Interaction(**request.json)

        if interaction.type == InteractionType.PING:
            return jsonify(InteractionResponse(InteractionResponseType.PONG).to_dict())
        elif interaction.type == InteractionType.APPLICATION_COMMAND:
            cmd = interaction.data.name
            cb = self._callbacks[cmd]

            cb_data = interaction
            if len(annotations := cb.__annotations__.values()) > 0:
                cmd_type = next(iter(annotations))
                if issubclass(cmd_type, ocm.Command):
                    cb_data = cmd_type.wrap(interaction)

            resp = cb(cb_data)

            if isinstance(resp, InteractionResponse):
                interaction_response = resp
            elif resp is None:
                interaction_response = InteractionResponse(
                    response_type=InteractionResponseType.ACKNOWLEDGE_WITH_SOURCE
                )
            else:
                interaction_response = InteractionResponse(
                    response_type=InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                    data=InteractionApplicationCommandCallbackData(content=str(resp)),
                )

            return jsonify(interaction_response.to_dict())

        else:
            return "Unknown interaction type", 501

    def register_command(
        self,
        command: Union[ApplicationCommand, Type[ocm.Command], str],
        callback: _CommandCallback,
    ):
        if isinstance(command, ApplicationCommand):
            self._commands[command.name] = command
            self._callbacks[command.name] = callback
        elif issubclass(command, ocm.Command):
            self._commands[command.__cmd_name__] = command.to_application_command()
            self._callbacks[command.__cmd_name__] = callback
        else:
            self._callbacks[command] = callback

    def command(
        self,
        command: Union[ApplicationCommand, str, _CommandCallback] = None,
    ):
        """
        A decorator to register a slash command.
        Calls :meth:`register_command` internally.
        """

        _f = None
        if isinstance(command, Callable):
            _f = command
            command = None

        def decorator(f: _CommandCallback):
            if command is not None:
                self.register_command(command, f)
            elif len(annotations := f.__annotations__.values()) > 0:
                _command = next(
                    iter(annotations)
                )  # get :class:`ocm.Command` from type annotation
                self.register_command(_command, f)
            else:
                self.register_command(f.__name__.lower().strip("_"), f)

        return decorator(_f) if _f is not None else decorator



#!/usr/bin/env python

"""
MIT License

Copyright (c) 2020-2021 Linus Bartsch

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from requests import Session, Request, Response, exceptions
from typing import List, Union, Type

from .application_command import ApplicationCommand as Cmd
from .interaction import Interaction
from .interaction_response import (
    InteractionResponse,
    InteractionApplicationCommandCallbackData,
    FollowupMessage,
)
from . import ocm

CmdClass = Type[ocm.Command]

API_BASE_URL = "https://discord.com/api/v8"


class _BaseClient:
    BASE_URL: str

    def __init__(self, app_id: int):
        self._s = Session()
        self._app_id = app_id

    def _app_url(self, *path) -> str:
        path = "/".join([str(p).strip("/") for p in path if p is not None])
        return f"{self.BASE_URL}/{self._app_id}/{path}"

    def _send(self, req: Request) -> Response:
        r = self._s.send(self._s.prepare_request(req))

        if not r.ok:
            raise exceptions.HTTPError(
                f"failed with status code {r.status_code}: {r.reason}: {r.json()}",
                response=r,
            )

        return r


class ApplicationClient(_BaseClient):
    BASE_URL = f"{API_BASE_URL}/applications"

    def __init__(self, token: str, app_id: int = None):
        super().__init__(app_id)
        self._token = token

        self._s.headers.update(self._auth_header)

        if app_id is None:
            r = self._send(Request("GET", f"{API_BASE_URL}/users/@me"))
            self._app_id = int(r.json()["id"])

    @property
    def _auth_header(self) -> dict:
        return {"Authorization": f"Bot {self._token}"}

    @property
    def application_id(self) -> int:
        return self._app_id

    def _cmd_url(self, cmd_id=None, guild_id=None):
        if guild_id is None:
            return self._app_url("commands", cmd_id)
        else:
            return self._app_url("guilds", guild_id, "commands", cmd_id)

    def get_commands(self, guild: int = None) -> List[Cmd]:
        """ Get all global or guild application commands. """

        r = self._send(Request("GET", self._cmd_url(guild_id=guild)))

        return [Cmd.from_dict(cmd) for cmd in r.json()]

    def create_command(self, cmd: Union[Cmd, CmdClass], guild: int = None) -> Cmd:
        """ Create a global or guild application command. """

        if not isinstance(cmd, Cmd):
            cmd = cmd.to_application_command()

        r = self._send(
            Request("POST", self._cmd_url(guild_id=guild), json=cmd.to_dict())
        )

        return Cmd.from_dict(r.json())

    def edit_command(self, cmd: Union[Cmd, CmdClass], guild: int = None) -> Cmd:
        """ Edit a global or guild application command. """

        if not isinstance(cmd, Cmd):
            cmd = cmd.to_application_command()

        if cmd.id is None:
            # creating a command with a name that already exist, overwrites the old one
            return self.create_command(cmd)

        r = self._send(
            Request("PATCH", self._cmd_url(cmd.id, guild), json=cmd.to_dict())
        )

        return Cmd.from_dict(r.json())

    def delete_command(self, cmd_id: int, guild: int = None):
        """ Delete a global or guild application command. """

        self._send(Request("DELETE", self._cmd_url(cmd_id, guild)))


class InteractionClient(_BaseClient):
    BASE_URL = f"{API_BASE_URL}/webhooks"

    def __init__(self, app_id: int, interaction: Interaction):
        super().__init__(app_id)

        self._interaction = interaction

    def _url(self, *path):
        return self._app_url(self._interaction.token, *path)

    def create_response(self, resp: InteractionResponse):
        """ Create a response to an interaction from the gateway. """

        url = "{0}/interactions/{1.id}/{1.token}/callback".format(
            API_BASE_URL,
            self._interaction,
        )

        self._send(Request("POST", url, json=resp.to_dict()))

    def edit_response(self, data: InteractionApplicationCommandCallbackData):
        """ Edit the initial Interaction response. """

        self._send(
            Request("PATCH", self._url("messages/@original"), json=data.to_dict())
        )

    def delete_response(self):
        """ Delete the initial Interaction response. """

        self._send(Request("DELETE", self._url("messages/@original")))

    def create_message(self, msg: FollowupMessage):
        """ Create a followup message for an Interaction. """

        self._send(Request("POST", self._url(), json=msg.to_dict()))

    def edit_message(self, msg_id: int, msg: FollowupMessage):
        """ Edit a followup message for an Interaction. """

        self._send(Request("PATCH", self._url("messages", msg_id), json=msg.to_dict()))

    def delete_message(self, msg_id: int):
        """ Delete a followup message for an Interaction. """

        self._send(Request("DELETE", self._url("messages", msg_id)))



#!/usr/bin/env python

"""
MIT License

Copyright (c) 2020-2021 Linus Bartsch

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import json
from enum import Enum
from .member import Member
from typing import Union


class InteractionType(Enum):
    PING = 1
    APPLICATION_COMMAND = 2


class _OptionGetter:
    options: list

    def get_option(
        self, option_name: str
    ) -> Union["ApplicationCommandInteractionDataOption", None]:
        """ Get option by name. """

        for option in self.options:
            if option.name == option_name:
                return option
        return None


class ApplicationCommandInteractionDataOption(_OptionGetter):
    def __init__(self, **kwargs):
        self.name = kwargs["name"]
        self.value = kwargs.get("value")
        self.options = [
            ApplicationCommandInteractionDataOption(**option)
            for option in kwargs.get("options", [])
        ]

    def __str__(self):
        return str(self.value)

    def __int__(self):
        return int(self.value)


class ApplicationCommandInteractionData(_OptionGetter):
    def __init__(self, **kwargs):
        self.id = int(kwargs["id"])
        self.name = kwargs["name"]
        self.options = [
            ApplicationCommandInteractionDataOption(**option)
            for option in kwargs.get("options", [])
        ]


INTERACTION_TYPE_MAP = {
    InteractionType.PING: type(None),
    InteractionType.APPLICATION_COMMAND: ApplicationCommandInteractionData,
}


class Interaction:
    """
    This represents the base interaction type that gets invoked for Slash Commands
    and future interaction types.
    """

    def __init__(self, **kwargs):
        self.id = int(kwargs["id"])
        self.type = InteractionType(kwargs["type"])

        if self.type == InteractionType.PING:
            return

        self.data = INTERACTION_TYPE_MAP[self.type](**kwargs.get("data", {}))
        self.guild_id = int(kwargs["guild_id"])
        self.channel_id = int(kwargs["channel_id"])
        self.member = Member(**kwargs["member"])
        self.token = kwargs["token"]
        self.version = int(kwargs["version"])

    @classmethod
    def from_json(cls, data: Union[dict, str]) -> "Interaction":
        if isinstance(data, str):
            data = json.loads(data)

        return cls(**data)



#!/usr/bin/env python

"""
MIT License

Original work Copyright (c) 2015-2020 Rapptz
Modified work Copyright (c) 2020-2021 Linus Bartsch

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from typing import List
from datetime import datetime
from enum import Enum


class UserFlags(Enum):
    staff = 1
    partner = 2
    hypesquad = 4
    bug_hunter = 8
    mfa_sms = 16
    premium_promo_dismissed = 32
    hypesquad_bravery = 64
    hypesquad_brilliance = 128
    hypesquad_balance = 256
    early_supporter = 512
    team_user = 1024
    system = 4096
    has_unread_urgent_messages = 8192
    bug_hunter_level_2 = 16384
    verified_bot = 65536
    verified_bot_developer = 131072


class PremiumType(Enum):
    nitro_classic = 1
    nitro = 2


class User:
    def __init__(self, **data):
        self.id = int(data["id"])
        self.username = data["username"]
        self.discriminator = data["discriminator"]
        self.avatar = data.get("avatar")
        self.bot = data.get("bot")
        self.system = data.get("system")
        self.mfa_enabled = data.get("mfa_enabled")
        self.locale = data.get("locale")
        self.verified = data.get("verified")
        self.email = data.get("email")
        self.flags = self._parse_flags(int(data.get("flags", 0)))
        self.premium_type = None

        if (premium_type := data.get("premium_type")) is not None:
            self.premium_type = PremiumType(premium_type)

        self.public_flags = self._parse_flags(int(data.get("public_flags", 0)))

    @staticmethod
    def _parse_flags(flags: int) -> List[UserFlags]:
        return [flag for flag in UserFlags if flags & flag.value != 0]

    @staticmethod
    def _flags_to_int(flags: List[UserFlags]) -> int:
        return sum(map(lambda flag: flag.value, flags))

    def to_dict(self) -> dict:
        data = {
            "id": str(self.id),
            "username": self.username,
            "discriminator": self.discriminator,
        }

        if self.avatar:
            data["avatar"] = self.avatar
        if self.bot:
            data["bot"] = self.bot
        if self.system:
            data["system"] = self.system
        if self.mfa_enabled:
            data["mfa_enabled"] = self.mfa_enabled
        if self.locale:
            data["locale"] = self.locale
        if self.verified:
            data["verified"] = self.verified
        if self.email:
            data["email"] = self.email
        if self.flags:
            data["flags"] = self._flags_to_int(self.flags)
        if self.premium_type:
            data["premium_type"] = self.premium_type.value
        if self.public_flags:
            data["public_flags"] = self._flags_to_int(self.public_flags)

        return data


class Member:
    def __init__(self, **data):
        self.user = None
        if user_data := data.get("user"):
            self.user = User(**user_data)

        self.nick = data["nick"]
        self.roles = data["roles"]
        self.joined_at = datetime.fromisoformat(data["joined_at"])
        self.premium_since = None

        if premium_since := data.get("premium_since"):
            self.premium_since = datetime.fromisoformat(premium_since)

        self.deaf = data["deaf"]
        self.mute = data["mute"]
        self.pending = data.get("pending", False)

    def to_dict(self) -> dict:
        data = {
            "nick": self.nick,
            "roles": self.roles,
            "joined_at": self.joined_at.isoformat(),
            "deaf": self.deaf,
            "mute": self.mute,
        }

        if self.user:
            data["user"] = self.user.to_dict()
        if self.premium_since:
            data["premium_since"] = self.premium_since.isoformat()
        if self.pending:
            data["pending"] = self.pending

        return data



#!/usr/bin/env python

"""
MIT License

Copyright (c) 2020-2021 Linus Bartsch

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from discord_interactions import (
    Interaction,
    Member,
    ApplicationCommandInteractionDataOption,
    ApplicationCommand,
    ApplicationCommandOption,
    ApplicationCommandOptionType,
    ApplicationCommandOptionChoice,
)
from typing import List, Union, Type
from dataclasses import dataclass
from enum import Enum


class OptionChoices(Enum):
    @classmethod
    def to_application_command_option_choices(
        cls,
    ) -> List[ApplicationCommandOptionChoice]:
        choices = []

        for choice in cls:
            choices.append(ApplicationCommandOptionChoice(choice.name, choice.value))

        return choices


class _Option:
    pass


class OptionContainerType(type):
    def __new__(mcs, name, bases, attributes):
        cls = super(OptionContainerType, mcs).__new__(mcs, name, bases, attributes)

        for attr_name, attr in attributes.items():
            if isinstance(attr, _Option):
                # set option name to attribute name where not explicitly set
                if attr.name is None:
                    attr.name = attr_name

                # set option type based on type annotations where not explicitly set
                if attr.type is None:
                    attr.type = (
                        ApplicationCommandOptionType.STRING
                    )  # use string as default option type
                    if cls.__annotations__:
                        _type = cls.__annotations__.get(attr_name)
                        if isinstance(_type, ApplicationCommandOptionType):
                            attr.type = _type
                        elif _type == int:
                            attr.type = ApplicationCommandOptionType.INTEGER
                        elif issubclass(_type, OptionChoices):
                            attr.choices = _type
                            _type = type(next(iter(_type)).value)
                            if _type == int:
                                attr.type = ApplicationCommandOptionType.INTEGER

        return cls


@dataclass()
class Option(_Option, metaclass=OptionContainerType):
    description: str
    type: ApplicationCommandOptionType = None
    name: str = None
    default: bool = False
    required: bool = False
    choices: Type[OptionChoices] = None
    __data: ApplicationCommandInteractionDataOption = None
    __data_loaded: bool = False

    @property
    def is_sub_command(self):
        return self.type in (
            ApplicationCommandOptionType.SUB_COMMAND,
            ApplicationCommandOptionType.SUB_COMMAND_GROUP,
        )

    def __get__(self, instance: Union["Command", "Option"], owner):
        if not self.is_sub_command:
            data = (
                getattr(instance, "_Command__interaction").data
                if not isinstance(self, owner)
                else instance.__data
            )

            if (option := data.get_option(self.name)) is not None:
                value = option.value
                return value if self.choices is None else self.choices(value)
            else:
                return None

        if not self.__data_loaded:
            if isinstance(self, owner):
                self.__data = instance.__data.get_option(self.name)
            else:
                self.__data = getattr(
                    instance, "_Command__interaction"
                ).data.get_option(self.name)
            self.__data_loaded = True
        return self.__data

    def to_application_command_option(self) -> ApplicationCommandOption:
        options = []
        for option in self.__dict__.values():
            if not isinstance(option, Option):
                continue
            options.append(option.to_application_command_option())

        if self.choices:
            choices = self.choices.to_application_command_option_choices()
        else:
            choices = None

        return ApplicationCommandOption(
            type=self.type,
            name=self.name,
            description=self.description,
            default=self.default,
            required=self.required,
            options=options or None,
            choices=choices,
        )


class CommandType(OptionContainerType):
    def __new__(mcs, *args, **kwargs):
        cls = super(CommandType, mcs).__new__(mcs, *args, **kwargs)

        # abort if it's a class in this module
        # (the `Command` class itself and not a subclass)
        if cls.__module__ == __name__:
            return cls

        if cls.__cmd_name__ is None:
            cls.__cmd_name__ = cls.__name__.lower().strip("_")

        if cls.__cmd_description__ is None and cls.__doc__ is not None:
            cls.__cmd_description__ = cls.__doc__.strip()

        return cls


class Command(metaclass=CommandType):
    """ Represents a Discord Slash Command in the Object-Command-Mapper (OCM). """

    __cmd_name__ = None
    __cmd_description__ = None

    __interaction: Interaction = None

    @classmethod
    def wrap(cls, interaction: Interaction):
        inst = cls()
        inst.__interaction = interaction
        return inst

    @property
    def guild_id(self) -> int:
        return self.__interaction.guild_id

    @property
    def channel_id(self) -> int:
        return self.__interaction.channel_id

    @property
    def member(self) -> Member:
        return self.__interaction.member

    @property
    def interaction_id(self) -> int:
        return self.__interaction.id

    @property
    def command_id(self) -> int:
        return self.__interaction.data.id

    @classmethod
    def to_application_command(cls) -> ApplicationCommand:
        options = []

        for option in cls.__dict__.values():
            if not isinstance(option, Option):
                continue
            options.append(option.to_application_command_option())

        return ApplicationCommand(
            name=cls.__cmd_name__,
            description=cls.__cmd_description__,
            options=options or None,
        )



from setuptools import setup, find_packages

with open('README.rst', 'r', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='discord-interactions.py',
    version='0.0.2',
    description='A library around the Discord Interactions API',
    long_description=readme,
    long_description_content_type='text/x-rst',
    url='https://github.com/LiBa001/discord-interactions.py',
    author='Linus Bartsch',
    author_email='pypi@libasoft.de',
    license='MIT',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries'
    ],
    python_requires='>=3.8',
    install_requires=['flask', 'pynacl', 'requests'],
    keywords='discord discord-py discord-bot wrapper',
    packages=find_packages(exclude=["examples", "tests"]),
    data_files=None
)



#!/usr/bin/env python

"""
MIT License

Copyright (c) 2020-2021 Linus Bartsch

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from enum import Enum
from typing import List, Protocol
from dataclasses import dataclass


class DictConvertible(Protocol):
    """
    Represents a generic dictionary convertible object type
    that can be implemented elsewhere.
    E.g. discord.py's `discord.Embed` or `discord.AllowedMentions` implements this.
    """

    def to_dict(self) -> dict:
        ...


class InteractionResponseType(Enum):
    PONG = 1
    ACKNOWLEDGE = 2
    CHANNEL_MESSAGE = 3
    CHANNEL_MESSAGE_WITH_SOURCE = 4
    ACKNOWLEDGE_WITH_SOURCE = 5


@dataclass()
class InteractionApplicationCommandCallbackData:
    content: str
    tts: bool = False
    embeds: List[DictConvertible] = None
    allowed_mentions: DictConvertible = None

    def to_dict(self) -> dict:
        data = {"content": str(self.content)}

        if self.tts:
            data["tts"] = self.tts
        if self.embeds:
            data["embeds"] = [embed.to_dict() for embed in self.embeds]
        if self.allowed_mentions:
            data["allowed_mentions"] = self.allowed_mentions.to_dict()

        return data


class InteractionResponse:
    def __init__(
        self,
        response_type: InteractionResponseType,
        data: InteractionApplicationCommandCallbackData = None,
    ):
        self.type = response_type
        self.data = data

    def to_dict(self) -> dict:
        response = {"type": self.type.value}

        if self.data:
            response["data"] = self.data.to_dict()

        return response


class FollowupMessage:
    content: str = None
    username: str = None
    avatar_url: str = None
    tts: bool = False
    embeds: List[DictConvertible] = None
    allowed_mentions: DictConvertible = None

    def to_dict(self) -> dict:
        data = {k: v for k, v in self.__dict__.items() if v}

        if self.content:
            data["content"] = str(self.content)
        if self.embeds:
            data["embeds"] = [embed.to_dict() for embed in self.embeds]
        if self.allowed_mentions:
            data["allowed_mentions"] = self.allowed_mentions.to_dict()

        return data
